package sun.awt.dnd;

import java.util.Arrays;
import sun.awt.datatransfer.ToolkitThreadBlockedHandler;
import java.util.HashSet;
import sun.awt.AppContext;
import sun.awt.SunToolkit;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.Point;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.Component;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.io.InputStream;
import sun.security.util.SecurityConstants;
import sun.awt.datatransfer.DataTransferer;
import java.awt.datatransfer.DataFlavor;
import java.awt.dnd.InvalidDnDOperationException;
import sun.util.logging.PlatformLogger;
import java.awt.dnd.DropTargetContext;
import java.awt.dnd.DropTarget;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.peer.DropTargetContextPeer;

public abstract class SunDropTargetContextPeer implements DropTargetContextPeer, Transferable
{
    public static final boolean DISPATCH_SYNC = true;
    private DropTarget currentDT;
    private DropTargetContext currentDTC;
    private long[] currentT;
    private int currentA;
    private int currentSA;
    private int currentDA;
    private int previousDA;
    private long nativeDragContext;
    private Transferable local;
    private boolean dragRejected;
    protected int dropStatus;
    protected boolean dropComplete;
    boolean dropInProcess;
    protected static final Object _globalLock;
    private static final PlatformLogger dndLog;
    protected static Transferable currentJVMLocalSourceTransferable;
    protected static final int STATUS_NONE = 0;
    protected static final int STATUS_WAIT = 1;
    protected static final int STATUS_ACCEPT = 2;
    protected static final int STATUS_REJECT = -1;
    
    public static void setCurrentJVMLocalSourceTransferable(Transferable currentJVMLocalSourceTransferable) throws InvalidDnDOperationException {
        synchronized (SunDropTargetContextPeer._globalLock) {
            if (currentJVMLocalSourceTransferable != null && SunDropTargetContextPeer.currentJVMLocalSourceTransferable != null) {
                throw new InvalidDnDOperationException();
            }
            SunDropTargetContextPeer.currentJVMLocalSourceTransferable = currentJVMLocalSourceTransferable;
        }
    }
    
    private static Transferable getJVMLocalSourceTransferable() {
        return SunDropTargetContextPeer.currentJVMLocalSourceTransferable;
    }
    
    public SunDropTargetContextPeer() {
        this.dragRejected = false;
        this.dropStatus = 0;
        this.dropComplete = false;
        this.dropInProcess = false;
    }
    
    @Override
    public DropTarget getDropTarget() {
        return this.currentDT;
    }
    
    @Override
    public synchronized void setTargetActions(int n) {
        this.currentA = (n & 0x40000003);
    }
    
    @Override
    public int getTargetActions() {
        return this.currentA;
    }
    
    @Override
    public Transferable getTransferable() {
        return this;
    }
    
    @Override
    public DataFlavor[] getTransferDataFlavors() {
        Transferable local = this.local;
        if (local != null) {
            return local.getTransferDataFlavors();
        }
        return DataTransferer.getInstance().getFlavorsForFormatsAsArray(this.currentT, DataTransferer.adaptFlavorMap(this.currentDT.getFlavorMap()));
    }
    
    @Override
    public boolean isDataFlavorSupported(DataFlavor dataFlavor) {
        Transferable local = this.local;
        if (local != null) {
            return local.isDataFlavorSupported(dataFlavor);
        }
        return DataTransferer.getInstance().getFlavorsForFormats(this.currentT, DataTransferer.adaptFlavorMap(this.currentDT.getFlavorMap())).containsKey(dataFlavor);
    }
    
    @Override
    public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException, IOException, InvalidDnDOperationException {
        SecurityManager securityManager = System.getSecurityManager();
        try {
            if (!this.dropInProcess && securityManager != null) {
                securityManager.checkPermission(SecurityConstants.AWT.ACCESS_CLIPBOARD_PERMISSION);
            }
        }
        catch (Exception ex) {
            Thread currentThread = Thread.currentThread();
            currentThread.getUncaughtExceptionHandler().uncaughtException(currentThread, ex);
            return null;
        }
        Transferable local = this.local;
        if (local != null) {
            return local.getTransferData(flavor);
        }
        if (this.dropStatus != 2 || this.dropComplete) {
            throw new InvalidDnDOperationException("No drop current");
        }
        Long n = (Long)DataTransferer.getInstance().getFlavorsForFormats(this.currentT, DataTransferer.adaptFlavorMap(this.currentDT.getFlavorMap())).get(flavor);
        if (n == null) {
            throw new UnsupportedFlavorException(flavor);
        }
        if (flavor.isRepresentationClassRemote() && this.currentDA != 1073741824) {
            throw new InvalidDnDOperationException("only ACTION_LINK is permissable for transfer of java.rmi.Remote objects");
        }
        long longValue = (long)n;
        Object nativeData = this.getNativeData(longValue);
        if ((nativeData instanceof byte[])) {
            try {
                return DataTransferer.getInstance().translateBytes((byte[])(byte[])nativeData, flavor, longValue, this);
            }
            catch (IOException ex3) {
                throw new InvalidDnDOperationException(ex3.getMessage());
            }
        }
        else if ((nativeData instanceof InputStream)) {
            try {
                return DataTransferer.getInstance().translateStream((InputStream)nativeData, flavor, longValue, this);
            }
            catch (IOException ex2) {
                throw new InvalidDnDOperationException(ex2.getMessage());
            }
        }
        else {
            throw new IOException("no native data was transfered");
        }
    }
    
    protected abstract Object getNativeData(long p0) throws IOException;
    
    @Override
    public boolean isTransferableJVMLocal() {
        return this.local != null || getJVMLocalSourceTransferable() != null;
    }
    
    private int handleEnterMessage(Component component, int n, int n2, int n3, int n4, long[] array, long n5) {
        return this.postDropTargetEvent(component, n, n2, n3, n4, array, n5, 504, true);
    }
    
    protected void processEnterMessage(SunDropTargetEvent sunDropTargetEvent) {
        Component component = (Component)sunDropTargetEvent.getSource();
        DropTarget dropTarget = component.getDropTarget();
        Point point = sunDropTargetEvent.getPoint();
        this.local = getJVMLocalSourceTransferable();
        if (this.currentDTC != null) {
            this.currentDTC.removeNotify();
            this.currentDTC = null;
        }
        if (component.isShowing() && dropTarget != null && dropTarget.isActive()) {
            this.currentDT = dropTarget;
            (this.currentDTC = this.currentDT.getDropTargetContext()).addNotify(this);
            this.currentA = dropTarget.getDefaultActions();
            try {
                dropTarget.dragEnter(new DropTargetDragEvent(this.currentDTC, point, this.currentDA, this.currentSA));
            }
            catch (Exception ex) {
                ex.printStackTrace();
                this.currentDA = 0;
            }
        }
        else {
            this.currentDT = null;
            this.currentDTC = null;
            this.currentDA = 0;
            this.currentSA = 0;
            this.currentA = 0;
        }
    }
    
    private void handleExitMessage(Component component, long n) {
        this.postDropTargetEvent(component, 0, 0, 0, 0, null, n, 505, true);
    }
    
    protected void processExitMessage(SunDropTargetEvent sunDropTargetEvent) {
        DropTarget dropTarget = ((Component)sunDropTargetEvent.getSource()).getDropTarget();
        if (dropTarget == null) {
            this.currentDT = null;
            this.currentT = null;
            if (this.currentDTC != null) {
                this.currentDTC.removeNotify();
            }
            this.currentDTC = null;
            return;
        }
        else {
            if (dropTarget != this.currentDT) {
                if (this.currentDTC != null) {
                    this.currentDTC.removeNotify();
                }
                this.currentDT = dropTarget;
                (this.currentDTC = dropTarget.getDropTargetContext()).addNotify(this);
            }
            DropTargetContext currentDTC = this.currentDTC;
            if (dropTarget.isActive()) {
                try {
                    dropTarget.dragExit(new DropTargetEvent(currentDTC));
                }
                catch (Exception ex) {
                    ex.printStackTrace();
                }
                finally {
                    this.currentA = 0;
                    this.currentSA = 0;
                    this.currentDA = 0;
                    this.currentDT = null;
                    this.currentT = null;
                    this.currentDTC.removeNotify();
                    this.currentDTC = null;
                    this.local = null;
                    this.dragRejected = false;
                }
            }
            return;
        }
    }
    
    private int handleMotionMessage(Component component, int n, int n2, int n3, int n4, long[] array, long n5) {
        return this.postDropTargetEvent(component, n, n2, n3, n4, array, n5, 506, true);
    }
    
    protected void processMotionMessage(SunDropTargetEvent sunDropTargetEvent, boolean b) {
        Component component = (Component)sunDropTargetEvent.getSource();
        Point point = sunDropTargetEvent.getPoint();
        sunDropTargetEvent.getID();
        DropTarget dropTarget = component.getDropTarget();
        if (component.isShowing() && dropTarget != null && dropTarget.isActive()) {
            if (this.currentDT != dropTarget) {
                if (this.currentDTC != null) {
                    this.currentDTC.removeNotify();
                }
                this.currentDT = dropTarget;
                this.currentDTC = null;
            }
            DropTargetContext dropTargetContext = this.currentDT.getDropTargetContext();
            if (dropTargetContext != this.currentDTC) {
                if (this.currentDTC != null) {
                    this.currentDTC.removeNotify();
                }
                (this.currentDTC = dropTargetContext).addNotify(this);
            }
            this.currentA = this.currentDT.getDefaultActions();
            try {
                DropTargetDragEvent dropTargetDragEvent = new DropTargetDragEvent(dropTargetContext, point, this.currentDA, this.currentSA);
                if (b) {
                    dropTarget.dropActionChanged(dropTargetDragEvent);
                }
                else {
                    dropTarget.dragOver(dropTargetDragEvent);
                }
                if (this.dragRejected) {
                    this.currentDA = 0;
                }
            }
            catch (Exception ex) {
                ex.printStackTrace();
                this.currentDA = 0;
            }
        }
        else {
            this.currentDA = 0;
        }
    }
    
    private void handleDropMessage(Component component, int n, int n2, int n3, int n4, long[] array, long n5) {
        this.postDropTargetEvent(component, n, n2, n3, n4, array, n5, 502, false);
    }
    
    protected void processDropMessage(SunDropTargetEvent sunDropTargetEvent) {
        Component component = (Component)sunDropTargetEvent.getSource();
        Point point = sunDropTargetEvent.getPoint();
        DropTarget dropTarget = component.getDropTarget();
        this.dropStatus = 1;
        this.dropComplete = false;
        if (component.isShowing() && dropTarget != null && dropTarget.isActive()) {
            DropTargetContext dropTargetContext = dropTarget.getDropTargetContext();
            this.currentDT = dropTarget;
            if (this.currentDTC != null) {
                this.currentDTC.removeNotify();
            }
            (this.currentDTC = dropTargetContext).addNotify(this);
            this.currentA = dropTarget.getDefaultActions();
            synchronized (SunDropTargetContextPeer._globalLock) {
                Transferable jvmLocalSourceTransferable = getJVMLocalSourceTransferable();
                this.local = jvmLocalSourceTransferable;
                if (jvmLocalSourceTransferable != null) {
                    setCurrentJVMLocalSourceTransferable(null);
                }
            }
            this.dropInProcess = true;
            try {
                dropTarget.drop(new DropTargetDropEvent(dropTargetContext, point, this.currentDA, this.currentSA, this.local != null));
            }
            finally {
                if (this.dropStatus == 1) {
                    this.rejectDrop();
                }
                else if (!this.dropComplete) {
                    this.dropComplete(false);
                }
                this.dropInProcess = false;
            }
        }
        else {
            this.rejectDrop();
        }
    }
    
    protected int postDropTargetEvent(Component target, int n, int n2, int n3, int n4, long[] array, long n5, int n6, boolean b) {
        AppContext targetToAppContext = SunToolkit.targetToAppContext(target);
        SunDropTargetContextPeer.EventDispatcher eventDispatcher = new SunDropTargetContextPeer.EventDispatcher(this, n3, n4, array, n5, b);
        SunDropTargetEvent event = new SunDropTargetEvent(target, n6, n, n2, eventDispatcher);
        if (b) {
            DataTransferer.getInstance().getToolkitThreadBlockedHandler().lock();
        }
        SunToolkit.postEvent(targetToAppContext, event);
        this.eventPosted(event);
        if (b) {
            while (!eventDispatcher.isDone()) {
                DataTransferer.getInstance().getToolkitThreadBlockedHandler().enter();
            }
            DataTransferer.getInstance().getToolkitThreadBlockedHandler().unlock();
            return eventDispatcher.getReturnValue();
        }
        return 0;
    }
    
    @Override
    public synchronized void acceptDrag(int n) {
        if (this.currentDT == null) {
            throw new InvalidDnDOperationException("No Drag pending");
        }
        this.currentDA = this.mapOperation(n);
        if (this.currentDA != 0) {
            this.dragRejected = false;
        }
        return;
    }
    
    @Override
    public synchronized void rejectDrag() {
        if (this.currentDT == null) {
            throw new InvalidDnDOperationException("No Drag pending");
        }
        this.currentDA = 0;
        this.dragRejected = true;
        return;
    }
    
    @Override
    public synchronized void acceptDrop(int n) {
        if (n == 0) {
            throw new IllegalArgumentException("invalid acceptDrop() action");
        }
        if (this.dropStatus == 1 || this.dropStatus == 2) {
            int mapOperation = this.mapOperation(n & this.currentSA);
            this.currentA = mapOperation;
            this.currentDA = mapOperation;
            this.dropStatus = 2;
            this.dropComplete = false;
            return;
        }
        throw new InvalidDnDOperationException("invalid acceptDrop()");
    }
    
    @Override
    public synchronized void rejectDrop() {
        if (this.dropStatus != 1) {
            throw new InvalidDnDOperationException("invalid rejectDrop()");
        }
        this.dropStatus = -1;
        this.currentDA = 0;
        this.dropComplete(false);
        return;
    }
    
    private int mapOperation(int n) {
        int[] array = { 2, 1, 1073741824 };
        int n2 = 0;
        int i = 0;
        while (i < array.length) {
            if ((n & array[i]) == array[i]) {
                n2 = array[i];
                break;
            }
            ++i;
        }
        return n2;
    }
    
    @Override
    public synchronized void dropComplete(boolean b) {
        if (this.dropStatus == 0) {
            throw new InvalidDnDOperationException("No Drop pending");
        }
        if (this.currentDTC != null) {
            this.currentDTC.removeNotify();
        }
        this.currentDT = null;
        this.currentDTC = null;
        this.currentT = null;
        this.currentA = 0;
        synchronized (SunDropTargetContextPeer._globalLock) {
            SunDropTargetContextPeer.currentJVMLocalSourceTransferable = null;
        }
        this.dropStatus = 0;
        this.dropComplete = true;
        try {
            this.doDropDone(b, this.currentDA, this.local != null);
        }
        finally {
            this.currentDA = 0;
            this.nativeDragContext = 0L;
        }
        return;
    }
    
    protected abstract void doDropDone(boolean p0, int p1, boolean p2);
    
    protected synchronized long getNativeDragContext() {
        return this.nativeDragContext;
    }
    
    protected void eventPosted(SunDropTargetEvent sunDropTargetEvent) {
    }
    
    protected void eventProcessed(SunDropTargetEvent sunDropTargetEvent, int n, boolean b) {
    }
    
    static {
        _globalLock = new Object();
        dndLog = PlatformLogger.getLogger("sun.awt.dnd.SunDropTargetContextPeer");
        SunDropTargetContextPeer.currentJVMLocalSourceTransferable = null;
    }
    
    protected static class EventDispatcher
    {
        private final SunDropTargetContextPeer peer;
        private final int dropAction;
        private final int actions;
        private final long[] formats;
        private long nativeCtxt;
        private final boolean dispatchType;
        private boolean dispatcherDone;
        private int returnValue;
        private final HashSet eventSet;
        static final ToolkitThreadBlockedHandler handler;
        
        EventDispatcher(SunDropTargetContextPeer peer, int dropAction, int actions, long[] original, long nativeCtxt, boolean dispatchType) {
            this.dispatcherDone = false;
            this.returnValue = 0;
            this.eventSet = new HashSet(3);
            this.peer = peer;
            this.nativeCtxt = nativeCtxt;
            this.dropAction = dropAction;
            this.actions = actions;
            this.formats = ((null == original) ? null : Arrays.copyOf(original, original.length));
            this.dispatchType = dispatchType;
        }
        
        void dispatchEvent(SunDropTargetEvent sunDropTargetEvent) {
            switch (sunDropTargetEvent.getID()) {
                case 504: {
                    this.dispatchEnterEvent(sunDropTargetEvent);
                    break;
                }
                case 506: {
                    this.dispatchMotionEvent(sunDropTargetEvent);
                    break;
                }
                case 505: {
                    this.dispatchExitEvent(sunDropTargetEvent);
                    break;
                }
                case 502: {
                    this.dispatchDropEvent(sunDropTargetEvent);
                    break;
                }
                default: {
                    throw new InvalidDnDOperationException();
                }
            }
        }
        
        private void dispatchEnterEvent(SunDropTargetEvent sunDropTargetEvent) {
            synchronized (this.peer) {
                this.peer.previousDA = this.dropAction;
                this.peer.nativeDragContext = this.nativeCtxt;
                this.peer.currentT = this.formats;
                this.peer.currentSA = this.actions;
                this.peer.currentDA = this.dropAction;
                this.peer.dropStatus = 2;
                this.peer.dropComplete = false;
                try {
                    this.peer.processEnterMessage(sunDropTargetEvent);
                }
                finally {
                    this.peer.dropStatus = 0;
                }
                this.setReturnValue(this.peer.currentDA);
            }
        }
        
        private void dispatchMotionEvent(SunDropTargetEvent sunDropTargetEvent) {
            synchronized (this.peer) {
                int n = this.peer.previousDA != this.dropAction;
                this.peer.previousDA = this.dropAction;
                this.peer.nativeDragContext = this.nativeCtxt;
                this.peer.currentT = this.formats;
                this.peer.currentSA = this.actions;
                this.peer.currentDA = this.dropAction;
                this.peer.dropStatus = 2;
                this.peer.dropComplete = false;
                try {
                    this.peer.processMotionMessage(sunDropTargetEvent, (boolean)n);
                }
                finally {
                    this.peer.dropStatus = 0;
                }
                this.setReturnValue(this.peer.currentDA);
            }
        }
        
        private void dispatchExitEvent(SunDropTargetEvent sunDropTargetEvent) {
            synchronized (this.peer) {
                this.peer.nativeDragContext = this.nativeCtxt;
                this.peer.processExitMessage(sunDropTargetEvent);
            }
        }
        
        private void dispatchDropEvent(SunDropTargetEvent sunDropTargetEvent) {
            synchronized (this.peer) {
                this.peer.nativeDragContext = this.nativeCtxt;
                this.peer.currentT = this.formats;
                this.peer.currentSA = this.actions;
                this.peer.currentDA = this.dropAction;
                this.peer.processDropMessage(sunDropTargetEvent);
            }
        }
        
        void setReturnValue(int returnValue) {
            this.returnValue = returnValue;
        }
        
        int getReturnValue() {
            return this.returnValue;
        }
        
        boolean isDone() {
            return this.eventSet.isEmpty();
        }
        
        void registerEvent(SunDropTargetEvent sunDropTargetEvent) {
            SunDropTargetContextPeer.EventDispatcher.handler.lock();
            if (!this.eventSet.add(sunDropTargetEvent) && SunDropTargetContextPeer.dndLog.isLoggable(PlatformLogger.Level.FINE)) {
                SunDropTargetContextPeer.dndLog.fine("Event is already registered: " + sunDropTargetEvent);
            }
            SunDropTargetContextPeer.EventDispatcher.handler.unlock();
        }
        
        void unregisterEvent(SunDropTargetEvent o) {
            SunDropTargetContextPeer.EventDispatcher.handler.lock();
            try {
                if (!this.eventSet.remove(o)) {
                    return;
                }
                if (this.eventSet.isEmpty()) {
                    if (!this.dispatcherDone && this.dispatchType) {
                        SunDropTargetContextPeer.EventDispatcher.handler.exit();
                    }
                    this.dispatcherDone = true;
                }
            }
            finally {
                SunDropTargetContextPeer.EventDispatcher.handler.unlock();
            }
            try {
                this.peer.eventProcessed(o, this.returnValue, this.dispatcherDone);
            }
            finally {
                if (this.dispatcherDone) {
                    this.nativeCtxt = 0L;
                    this.peer.nativeDragContext = 0L;
                }
            }
        }
        
        public void unregisterAllEvents() {
            Object[] array = null;
            SunDropTargetContextPeer.EventDispatcher.handler.lock();
            try {
                array = this.eventSet.toArray();
            }
            finally {
                SunDropTargetContextPeer.EventDispatcher.handler.unlock();
            }
            if (array != null) {
                for (int i = 0; i < array.length; ++i) {
                    this.unregisterEvent((SunDropTargetEvent)array[i]);
                }
            }
        }
        
        static {
            handler = DataTransferer.getInstance().getToolkitThreadBlockedHandler();
        }
    }
}
